#!/usr/bin/env python

# Support functions for querying .tel domains

# Copyright (c) 2009, Andrew McDonald
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * The name of the author may not be used to endorse or promote
#      products derived from this software without specific prior
#      written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import re
import csv
import base64
import shlex
import dns.resolver
from M2Crypto import RSA

def split_regexp(regexp):
    """
    Split regular expression using delimiter, and taking escaped
    delimiters into account. Returns a list containing the two
    parts of the regex.
    """
    delim = regexp[0]

    # use shlex rather than split to handle escaped delimiters
    rex_splitter = shlex.shlex(regexp, posix=True)
    rex_splitter.commenters = ''
    rex_splitter.whitespace = delim
    rex_splitter.whitespace_split = True
    rex_splitter.escape = '\\'
    rex_splitter.quotes = ''
    rex_splitter.escapedquotes = ''

    rexbits = list(rex_splitter)

    if len(rexbits) != 2:
        raise "problem handling naptr regexp"
    if rexbits[0] != "^.*$":
        #print "warning: naptr handling needs fixing"
        pass

    res = rexbits[1]
    return res

def loclbl(service):
    """
    Convert service into human readable string.
    """
    lbl = ""
    if re.search("\+x-mobile", service):
        lbl = "Mobile"
    elif re.search("\+x-work", service):
        lbl = "Work"
    elif re.search("\+x-main", service):
        lbl = "Main Contact"
    elif re.search("\+x-home", service):
        lbl = "Home"
    elif re.search("\+x-transit", service):
        lbl = "In Transit"
    elif re.search("\+x-prs", service):
        lbl = "Premium Rate"

    m = re.search(r"\+x-lbl:(.*)", service)
    if m:
        # FIXME: doesn't handle escaped characters
        l = m.group(1).split("+")[0]
    else:
        l = None
    if l and lbl:
        lbl += ' '
    if l:
        lbl += '('+l+')'

    return lbl

def query(teldomain, service="", pk=None, friends=None):
    """
    Query for service within teldomain. Will use private key and
    friends list to access private data if these are supplied.
    """
    if friends and teldomain in friends:
        lookup = friends[teldomain]+'.'+teldomain
    else:
        lookup = teldomain

    r = dns.resolver.get_default_resolver()
    # FIXME: handle tcp fallback
    r.use_edns(0,0,4096)
    try:
        answers = r.query(lookup, 'NAPTR')
    except dns.resolver.NXDOMAIN:
        return []
    urilist = []
    for rdata in answers:
        if pk and re.match(re.escape("E2U+x-crypto:data:8210"), rdata.service):
            crypdata = split_regexp(rdata.regexp)
            if re.match(re.escape("data:;base64,"), crypdata[1]):
                raise "unhandled x-crypto"
            encdata = base64.b64decode(crypdata[13:])
            decnaptr = pk.private_decrypt(encdata,RSA.pkcs1_padding)
            naptrdata = []
            while len(decnaptr) > 0:
                l = ord(decnaptr[0])
                naptrdata.append(decnaptr[1:1+l])
                decnaptr = decnaptr[l+1:]
            if re.match(re.escape("E2U+"+service), naptrdata[1]):
                uri = split_regexp(naptrdata[2])
                lbl = loclbl(naptrdata[1])
                urilist.append((rdata.order, rdata.preference, uri, lbl))

        elif re.match(re.escape("E2U+"+service), rdata.service):
            uri = split_regexp(rdata.regexp)
            lbl = loclbl(rdata.service)
            urilist.append((rdata.order, rdata.preference, uri, lbl))

    return urilist

def nametxt(teldomain):
    """
    Look up the natural name for a teldomain. Returns tuple containing
    the name and boolean to indicate if private data is present.
    """
    r = dns.resolver.get_default_resolver()
    # FIXME: handle tcp fallback
    r.use_edns(0,0,4096)

    try:
        answers = r.query(teldomain, 'TXT')
    except dns.resolver.NXDOMAIN:
        return ("", False)
    defname = None
    name = None
    privdata = None
    for rdata in answers:
        if re.match("\.t", rdata.strings[0]):
            if rdata.strings[0] == '.tkw' and \
                    rdata.strings[1] == '1' and \
                    rdata.strings[2] == 'nl':
                i = 4
                fn = None
                ln = None
                while i < len(rdata.strings):
                    if rdata.strings[i] == 'fn':
                        fn = rdata.strings[i+1]
                    elif rdata.strings[i] == 'ln':
                        ln = rdata.strings[i+1]
                    i += 2
                if fn and ln:
                    name = fn+' '+ln
                elif fn:
                    name = fn
                elif ln:
                    name = ln
                if name:
                    break
            elif rdata.strings[0] == '.tsm' and \
                    rdata.strings[1] == '1' and \
                    rdata.strings[2] == 'pddx':
                if rdata.strings[3] == '1':
                    privdata = False
                elif rdata.strings[3] == '0':
                    privdata = True
        else:
            defname = rdata.strings[0]


    if not name:
        if defname:
            name = defname
        else:
            name = ""

    return (name, privdata)

def printemaillist(teldomain, pk=None, friends=None):
    """
    Look up and print email addresses in teldomain, using
    private key and friends list if supplied.
    """
    emaillist = query(teldomain, "email:mailto", pk, friends)
    (name, privdata) = nametxt(teldomain)
    if emaillist:
        if len(emaillist) > 1:
            pl = "es"
        else:
            pl = ""
        print "%s found %d e-mail address%s:" % (teldomain, len(emaillist), pl)
        emaillist.sort()
        for (_,_,e,l) in emaillist:
            e = re.sub("^mailto:", "", e)
            if l == None:
                l = ""
            print "%s\t%s\t%s" % (e,name,l)
    else:
        if name:
            n = " (%s)" % name
        else:
            n = ""
        if privdata:
            p = " but private data exists - try friending"
        else:
            p = ""
        print "%s%s has no-email addresses%s" % (teldomain, n, p)

def printlist(teldomain, service=None, pk=None, friends=None):
    """
    Look up and print entries in teldomain, using
    private key and friends list if supplied.
    """
    if service == None:
        service = ""

    resultlist = query(teldomain, service, pk, friends)
    (name, privdata) = nametxt(teldomain)
    if name:
        print "%s - %s" % (teldomain, name)
    else:
        print "%s" % teldomain
    if resultlist:
        resultlist.sort()
        for (_,_,u,l) in resultlist:
            if l == None:
                l = ""
            print "%s\t%s" % (u,l)

    if not teldomain in friends and privdata:
        print "Private data may exist - try friending"

if __name__ == "__main__":

    pk = RSA.load_key("telkey.pem")

    friends = {}
    f = open("friendlist.csv")
    csv_reader = csv.reader(f)
    for r in csv_reader:
        friends[r[0]] = r[1]

    query(sys.argv[1], pk, friends)


