#!/usr/bin/env python

# SOAP support functions for .tel telfriends functionality

# Copyright (c) 2009, Andrew McDonald
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * The name of the author may not be used to endorse or promote
#      products derived from this software without specific prior
#      written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import urllib2
import base64
import xml.dom.minidom
from M2Crypto import RSA

initurl = "https://soap.telfriends.tel/init?wsdl"
memburl = "https://soap.telfriends.tel/member?wsdl"

def getText(nodelist):
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc

def extractChallQuestion(stream):
    dom = xml.dom.minidom.parse(stream)
    chall = getText(dom.getElementsByTagName("ns2:challengeQuestion")[0].childNodes)
    return chall

def extractAPICredentials(stream, password):
    dom = xml.dom.minidom.parse(stream)
    soid = getText(dom.getElementsByTagName("ns2:soid")[0].childNodes)
    enc_api_pass = getText(dom.getElementsByTagName("ns2:apiPassword")[0].childNodes)
    enc_private_key = getText(dom.getElementsByTagName("privateKey")[0].childNodes)
    epk = ""
    while len(enc_private_key) > 0:
        epk = epk + '\n'
        epk = epk + enc_private_key[:76]
        if len(enc_private_key) > 76:
            enc_private_key = enc_private_key[76:]
        else:
            enc_private_key = ""
    pem_private_key = str("-----BEGIN ENCRYPTED PRIVATE KEY-----"+epk+"\n-----END ENCRYPTED PRIVATE KEY-----\n")
    pk = RSA.load_key_string(pem_private_key, lambda *args: password)
    api_pass = pk.private_decrypt(base64.decodestring(enc_api_pass),
                                  RSA.pkcs1_padding)

    return (soid, api_pass, pk)

def extractPublishers(stream):
    dom = xml.dom.minidom.parse(stream)
    entries = dom.getElementsByTagName("ns3:entry")
    pubs = []
    for e in entries:
        pub = getText(e.getElementsByTagName("ns3:publisher")[0].childNodes)
        label = getText(e.getElementsByTagName("ns3:label")[0].childNodes)
        pubs.append((pub,label))
    return pubs

def getChallenge(username, password):
    authdata = base64.encodestring('%s:%s' % (username, password))[:-1]
    authheader =  "Basic %s" % authdata

    req = urllib2.Request(initurl)
    req.add_header("User-Agent", "PyTel/1.0")
    req.add_header("Authorization", authheader)
    req.add_header("Content-Type", "application/soap+xml; charset=utf-8")

    req.add_data('<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope" xmlns:typ="http://xmlns.telnic.org/ws/so/init/types-1.0"><soap:Header/><soap:Body>'+\
                     '<getChallengeQuestionRequest xmlns="http://xmlns.telnic.org/ws/so/init/types-1.0"/>'+\
                     '</soap:Body></soap:Envelope>')

    try:
        response = urllib2.urlopen(req)
    except urllib2.HTTPError, e:
        if e.code == 401:
            return None
        raise e
    return extractChallQuestion(response)

def getAPICredentials(username, password, challresp):
    authdata = base64.encodestring('%s:%s' % (username, password))[:-1]
    authheader =  "Basic %s" % authdata

    req = urllib2.Request(initurl)
    req.add_header("User-Agent", "PyTel/1.0")
    req.add_header("Authorization", authheader)
    req.add_header("Content-Type", "application/soap+xml; charset=utf-8")

    req.add_data('<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope" xmlns:typ="http://xmlns.telnic.org/ws/so/init/types-1.0"><soap:Header/><soap:Body>' + \
                     '<getAPICredentialsRequest xmlns="http://xmlns.telnic.org/ws/so/init/types-1.0"><challengeAnswer>%s</challengeAnswer>' % challresp + \
                     '</getAPICredentialsRequest></soap:Body></soap:Envelope>')

    try:
        response = urllib2.urlopen(req)
    except urllib2.HTTPError, e:
        if e.code == 401:
            return None
        if e.code == 500:
            # probably ought to look at the xml passed back
            return None
        raise e
    return extractAPICredentials(response, password)

def listPublishers(username, password):
    authdata = base64.encodestring('%s:%s' % (username, password))[:-1]
    authheader =  "Basic %s" % authdata

    req = urllib2.Request(memburl)
    req.add_header("User-Agent", "PyTel/1.0")
    req.add_header("Authorization", authheader)
    req.add_header("Content-Type", "application/soap+xml; charset=utf-8")

    req.add_data('<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope" xmlns:typ="http://xmlns.telnic.org/ws/so/member/publisherstore/types-1.0"><soap:Header/><soap:Body>'+\
                     '<listPublishersRequest xmlns="http://xmlns.telnic.org/ws/so/member/publisherstore/types-1.0"/>'+\
                     '</soap:Body></soap:Envelope>')

    try:
        response = urllib2.urlopen(req)
    except urllib2.HTTPError, e:
        if e.code == 401:
            return None
        raise e
    return extractPublishers(response)

def main(username, password, challengeresp):
    chall = getChallenge(username, password)
    print "Challenge Question: %s" % chall
    (soid, api_pass, priv_key) = getAPICredentials(username, password, challengeresp)
    print soid
    print api_pass
    print priv_key
    pubs = listPublishers(soid, api_pass)
    print pubs

    priv_key.save_pem("telkey.pem", None)
    csvfile = open("friendlist.csv", "w+")
    for (p,l) in pubs:
        csvfile.write("%s,%s\n" % (p,l))
    csvfile.close()

if __name__ == "__main__":
    main(sys.argv[1], sys.argv[2], sys.argv[3])
